import matplotlib.pyplot as plt
from scipy.signal import freqz
import numpy as np


SR = 16000
BD = 50
F_SPACE = 1000
F_MARK = 2000
IDLEBITS = 5
MESSAGELEN = 7


def set_constants(sr, bd, f_space, f_mark, idlebits, messagelen):
    global SR, BD, F_SPACE, F_MARK, IDLEBITS, MESSAGELEN
    SR = sr
    BD = bd
    F_SPACE = f_space
    F_MARK = f_mark
    IDLEBITS = idlebits
    MESSAGELEN = messagelen


def plot_signal(signal, xlim=(200, 400), ylim=(-1.5, 1.5)):
    plt.plot(signal, lw=2)
    if xlim:
        plt.xlim(xlim)
    if ylim:
        plt.ylim(ylim)
    plt.xlabel("Time")
    plt.ylabel("Magnitude")


def plot_freq(signal, flim=4000, xlim=None,):
    plt.specgram(signal, NFFT=128, noverlap=64, Fs=SR)
    plt.ylim(0, flim)
    if xlim:
        plt.xlim(xlim)


def plot_filter(h, flim=4000):
    w, mag = freqz(h)
    plt.plot(w * SR/4, np.abs(mag), lw=2)
    plt.xlim(0, flim)


def plot_decoder(det):
    nbits = 8 * MESSAGELEN
    plt.figure(figsize=(16, 12))
    plt.plot(det, color='b', lw=2)
    ymax = det[(SR/BD)/2::SR/BD]
    plt.vlines((SR/BD)/2 + SR/BD * np.arange(nbits + IDLEBITS), 0, ymax)


def make_rcf_filter(rcf_b=0.5, rcf_w=3):
    t = np.linspace(-rcf_w/BD, rcf_w/BD, SR/BD * 2 * rcf_w)
    rcf = np.sin(np.pi * t * BD) / (np.pi * t * BD)
    rcf *= np.cos(np.pi * rcf_b * t * BD)
    rcf /= (1 - (2 * rcf_b * t * BD)**2)
    return rcf


def string_to_bits(string):
    b = [int(b) for b in ''.join(format(ord(i), 'b').zfill(8) for i in string)]
    return b


def bits_to_string(b):
    return "".join(
        chr(int("".join(map(str, b[i:i+8])), 2)) for i in range(0, len(b), 8))


def sample_det(det, rcf_w=3):
    return ((det[(SR/BD)/2::SR/BD] > 0) * 1)[rcf_w:-(IDLEBITS - rcf_w)]


def modulate_bits(bits):
    bitfreqs = [F_SPACE + bit * (F_MARK - F_SPACE) for bit in bits]
    freqs = np.repeat(bitfreqs, SR / BD)
    signal = np.sin(2*np.pi*np.cumsum(freqs)/SR)
    signal = np.hstack((signal, 0.01 * np.ones(IDLEBITS * SR / BD)))
    return signal
